/* **********************************************************
*															*
*	1-Kanal Universalrelais -regler							*
*															*
*	Funktionsumfang:										*
*	Blinker (Blinkzeit: Bereich 0,1s - 25,6s)				*
*	Stromstorelais (Flip-Flop Funktion)					*
*	Zeitrelais (Abschaltverzgerung: 0,1s - 25,6s)			*
*	Zweipunktregler 										*
*															*
*	Prozessor: AtTiny 13									*
********************************************************** */

#include <avr/io.h>
#include <avr/interrupt.h>

//Defines
#define OUTPUT_LOW	0x03	//PB0, PB1 Pull UP, PB2 = 0, PB3, PB4 kein Pull Up
#define OUTPUT_HIGH	0x07	//PB0, PB1 Pull UP, PB2 = 1, PB3, PB4 kein Pull Up
#define RELAIS		2		//Relais auf PB2 (Bit 2) angeschlossen
#define TOIE0		1		//Timer 0 Interrupt Freigabe Bit
#define TIME_10MS	68		//Ladewert Timer 0 fr 10ms
#define MAXPRELL	3		//max. 30ms Prellzeit
#define ADC2		0x02	//ADC Kanal 2 (Referenzeingang)
#define ADC3		0x03	//ADC Kanal 3 (Messeingang)
#define HYSTERESE   20		//Hysterese (+-20 Count --> +-100mV)

//Function declaration
void InitFunktion(void);
unsigned char AD_Wandler_8Bit(unsigned char byKanalNr);
unsigned int AD_Wandler_10Bit(unsigned char byKanalNr);

//Globales
unsigned char byFlipFlop;		//Flip Flop
unsigned char byToggelFF;		//Toggel Flip Flop
unsigned char byMonoFlop;		//Monoflop
unsigned char byTime_10MS;		//10ms Flag
unsigned char byPrellz;			//Prellzaehler
unsigned char byTimer_100MS;	//100ms Timer
unsigned char byMonoZeit;
unsigned char byZeit;
unsigned int iReglerRef;		//Regler Referenzwert
unsigned int iReglerUx;			//Regler Istwert

ISR(TIM0_OVF_vect)
{
	TCNT0 = TIME_10MS;				//Tiner Nachladen fr 10ms
	byTime_10MS = 1;				//10 ms abgelaufen
	byTimer_100MS++;
	if(byTimer_100MS > 10)
	{
		byTimer_100MS = 0;
		if(byMonoFlop == 1)
		{								//Monoflop luft
			byMonoZeit--;
			if(byMonoZeit == 0)
			{
				byMonoFlop = 0;			//Monflop abgelaufen
			}
		}
		if(byToggelFF == 1)
		{								//Toggel Flip Flop Funktion
			byMonoZeit--;
			if(byMonoZeit == 0)
			{
				byMonoZeit = byZeit;
				byFlipFlop++;			//Komplement Flip Flop
				byFlipFlop = byFlipFlop & 0x01;
			}
		}
	}
}

			
int main(void)
{
	unsigned char byMode, byModeAlt;

	DDRB = 0x04;		//x x x PB4:In  PB3:In  PB2:Out  PB1:In  PB0:In
	PORTB= OUTPUT_LOW;	//PB0, PB1 Pull UP, PB2 = 0, PB3, PB4 kein Pull Up

	byMode = PINB;		//aktuelle Betriebsart laden
	byMode = byMode & 0x03;
	byModeAlt = byMode;
	InitFunktion();		//Initialisierung


	while (1)
	{
		byMode = PINB;
		byMode = byMode & 0x03;
		if(byMode != byModeAlt)
		{	//Betriebsart hat sich gendert
			byModeAlt = byMode;
			InitFunktion();
		}
		else
		{
			if(byMode == 0x00)
			{
				byZeit = AD_Wandler_8Bit(ADC3);	//AD-Wandlung Kanal Potentiometer
				if(byZeit == 0)
				{
					byZeit++;					//Zeit garantiert > 0
				}
				//Blinker
				if((PINB & 0b00010000) != 0)
				{
					//Blinker EIN
					byToggelFF = 1;	//Toggel Flip Flop einschalten
					if(byFlipFlop == 0x01)
					{
						PORTB = OUTPUT_LOW;		//Relais Aus
					}
					else
					{
						PORTB = OUTPUT_HIGH;	//Relais Ein
					}
				}
				else
				{
					byToggelFF = 0;		//Toggel Flip Flop ausschalten
					byFlipFlop = 0;
					PORTB = OUTPUT_LOW;	//Relais Aus
					byMonoZeit = byZeit;
				}
			}

			if(byMode == 0x01)
			{
				//Stromstossrelais
				if(byTime_10MS == 1)
				{
					byTime_10MS = 0;
					if((PINB & 0b00010000) != 0)
					{
						if(byPrellz != 0)
						{
							byPrellz--;
							if(byPrellz == 0)
							{
								byFlipFlop++;			//Komplement FlipFlop
								byFlipFlop = byFlipFlop & 0x01;
								if(byFlipFlop == 1)
								{
									PORTB = OUTPUT_HIGH;	//Relais Ein
								}
								else
								{
									PORTB = OUTPUT_LOW;		//Relais Aus
								}
							}
						}
					}
					else
					{
						byPrellz = MAXPRELL;	//Prellzhler nachladen
					}
				}
			}

			if(byMode == 0x02)
			{
				byZeit = AD_Wandler_8Bit(ADC3);	//AD-Wandlung Kanal Potentiometer
				if(byZeit == 0)
				{
					byZeit++;					//Zeit garantiert > 0
				}
				//Zeitrelais
				if((PINB & 0b00010000) != 0)
				{
					//Monflop EIN
					byMonoFlop = 1;				//Monoflop einschalten
					PORTB = OUTPUT_HIGH;		//Relais Ein
					byMonoZeit = byZeit;
				}
				else
				{
					if(byMonoFlop != 1)
					{
						PORTB = OUTPUT_LOW;		//Relais Aus
					}
				}
			}

			if(byMode == 0x03)
			{
				//Regler
				iReglerRef = AD_Wandler_10Bit(ADC3);	//Sollwert (Poti) messen
				if(iReglerRef > 950)
				{
					iReglerRef = 950;					//Sollwert begrenzen
				}
				if(iReglerRef < 70)
				{
					iReglerRef = 70;
				}
				iReglerUx = AD_Wandler_10Bit(ADC2);		//Istwert (Eingang) messen
				if(iReglerUx > (iReglerRef + HYSTERESE))
					PORTB = OUTPUT_LOW;		//Relais Aus
				if(iReglerUx < (iReglerRef - HYSTERESE))
					PORTB = OUTPUT_HIGH;	//Relais Ein
			}
		}
	}
}

void InitFunktion(void)
{
	cli();					//Interrupt sperren fr Initialisierung
	PORTB = OUTPUT_LOW;		//Relaisausgang abschalten
	TCNT0 = TIME_10MS;		//Tiner SET fr 10ms = 1/(1200kHz/64) * (256-TIME_10MS)
	TCCR0B = 0x03;			//Timer 0: Vorteiler 64 u. Timer starten
	byTime_10MS = 0;		//Steuerflag und Variablen initalisieren
	byTimer_100MS = 0;
	byFlipFlop = 0;
	byToggelFF = 0;
	byMonoFlop = 0;
	byPrellz = MAXPRELL;
	ADCSRA = 0b00000011;	//ADC-Takt = CPU Takt/8
	ADCSRA = 0b10000011;	//ADC Enable
	TIMSK0 |= (1 << TOIE0);	//Timer Interrupt
	sei();					//freischalten
}

unsigned char AD_Wandler_8Bit(unsigned char byKanalNr)
{
	unsigned char byN;

	byKanalNr = byKanalNr & 0x03;
	byKanalNr = byKanalNr | 0b00100000;	//8 bit Betriebsart
	ADMUX = byKanalNr;					//ADC Inputkanal whlen (Vref = Vcc)
	ADCSRA = 0b11000011;				//Start ADC
	byN = 0b01000000;
	while(byN == 0b01000000)
	{
		byN = ADCSRA;
		byN = byN & 0b01000000;
	}
	ADCSRA = 0b11000011;				//2. Start ADC (notwendig wegen Kanalwechsel)
	byN = 0b01000000;
	while(byN == 0b01000000)
	{
		byN = ADCSRA;
		byN = byN & 0b01000000;
	}

	byN = ADCH;							//AD-Wandler Ergebnis lesen.
			
	return(byN);
}


unsigned int AD_Wandler_10Bit(unsigned char byKanalNr)
{
	unsigned char byN;
	unsigned int iADC_Value;

	byKanalNr = byKanalNr & 0x03;
	ADMUX = byKanalNr;				//ADC Inputkanal whlen (Vref = Vcc)
	ADCSRA = 0b11000011;			//Start ADC
	byN = 0b01000000;
	while(byN == 0b01000000)
	{
		byN = ADCSRA;
		byN = byN & 0b01000000;
	}
	ADCSRA = 0b11000011;			//Start ADC
	byN = 0b01000000;
	while(byN == 0b01000000)
	{
		byN = ADCSRA;
		byN = byN & 0b01000000;
	}

	byN = ADCL;								//AD-Wandler Ergebnis lesen. Low Byte zuerst
	iADC_Value = ((unsigned int)ADCH);		//High Byte
	iADC_Value = iADC_Value << 8;
	iADC_Value = iADC_Value & 0x0300;
	iADC_Value = iADC_Value | ((unsigned int)byN);
		
	return(iADC_Value);
}

